#include "Console.h"
#include <time.h>
#include "..\resource.h"
#include "..\mainHelper.h"

#define MENU_ITEM_COPY   			1
#define MENU_ITEM_CLOSE				2
#define MENU_ITEM_CLEAR				3
HWND WindowConsole = NULL;

HMENU pupup_menu = NULL;


int iNext = 0;
const int iMaxEntries = 1000;

WNDPROC WndprocConsoleBackup = NULL;
LRESULT CALLBACK WndprocConsole(HWND hwnd, UINT message, WPARAM wp, LPARAM lp);

Console* pConsole;

bool bConsoleVisible;
WINDOWPLACEMENT WinPlaceConsole;

RECT rConsoleDefault =
{
	50, 50, 1000, 700
};

BOOL Console::Create()
{
	WindowConsole = CreateWindowEx(WS_EX_TOOLWINDOW |   						  // DWORD dwExStyle,
			WS_EX_CLIENTEDGE,				//
		TEXT("LISTBOX"),							 // LPCTSTR lpClassName
		TEXT("Console"),							 // LPCTSTR lpWindowName
		WS_VSCROLL |						// DWORD dwStyle
			LBS_DISABLENOSCROLL |   		//
			LBS_EXTENDEDSEL |   			//
			LBS_HASSTRINGS |				//
			LBS_NOTIFY |					//
			LBS_NOINTEGRALHEIGHT |  		//
			WS_POPUP |  					//
			WS_BORDER | 					//
			WS_CAPTION |					//
			WS_THICKFRAME | 				//
			WS_MAXIMIZEBOX,					//
		rConsoleDefault.left,   					   // int x
		rConsoleDefault.top,						   // int y
		rConsoleDefault.right - rConsoleDefault.left,  // int nWidth
		rConsoleDefault.bottom - rConsoleDefault.top,  // int nHeight
		NULL,   							// HWND hWndParent
		NULL,   									   // HMENU hMenu
		AfxGetInstanceHandle(), 								  // HINSTANCE hInstance
		NULL										   // LPVOID lpParam
	);
	SetWindowPos(WindowConsole, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_FRAMECHANGED);

	if (!WindowConsole)
		return FALSE;


	SendMessage(WindowConsole, LB_INSERTSTRING, 0, (LPARAM)TEXT(""));

	bConsoleVisible = (WinPlaceConsole.showCmd != SW_HIDE);
	SetWindowPlacement(WindowConsole, &WinPlaceConsole);


	// So we can always fully read the last line...
	SendMessage(WindowConsole, LB_INSERTSTRING, 0, (LPARAM)"");

	// Default font
HFONT hFont = CreateFont(-11,   					   // int nHeight
				0,  						  // int nWidth
				0,  						  // int nEscapement
				0,  						  // int nOrientation
				FW_REGULAR, 				  // int fnWeight
				FALSE,  					  // DWORD fdwItalic
				FALSE,  					  // DWORD fdwUnderline
				FALSE,  					  // DWORD fdwStrikeOut
				DEFAULT_CHARSET,				 // DWORD fdwCharSet
				OUT_TT_PRECIS,  			  // DWORD fdwOutputPrecision
				CLIP_DEFAULT_PRECIS,		  // DWORD fdwClipPrecision
				ANTIALIASED_QUALITY,		  // DWORD fdwQuality
				FF_DONTCARE | DEFAULT_PITCH,  // DWORD fdwPitchAndFamily
				TEXT("Verdana") 			// LPCTSTR lpszFace
		);
		::SendMessage(WindowConsole, WM_SETFONT, (WPARAM)hFont, FALSE);


	// Exchange window procedure
	WndprocConsoleBackup = (WNDPROC)GetWindowLong(WindowConsole, GWL_WNDPROC);
	if (WndprocConsoleBackup != NULL)
	{
		SetWindowLong(WindowConsole, GWL_WNDPROC, (LONG)WndprocConsole);
	}

	// Create menu
	pupup_menu = CreatePopupMenu();
	AppendMenu(pupup_menu, MF_STRING, MENU_ITEM_COPY, m_Helper.MyLoadString(IDS_CONSOLE_COPY));
	AppendMenu(pupup_menu, MF_STRING, MENU_ITEM_CLEAR, m_Helper.MyLoadString(IDS_CONSOLE_CLEAR));
	AppendMenu(pupup_menu, MF_STRING, MENU_ITEM_CLOSE, m_Helper.MyLoadString(IDS_CONSOLE_CLOSE));
	return TRUE;
}


BOOL Console::Destroy()
{
	if (!WindowConsole)
		return FALSE;
	DestroyWindow(WindowConsole);
	DestroyMenu(pupup_menu);

	return TRUE;
}


BOOL Console::Append(TCHAR* szText)
{
	if (!WindowConsole || AfxGetApp()->GetProfileInt(_T("Preferences"), _T("displayConsole"), 0) == 0)
		return FALSE;
	if (iNext > iMaxEntries - 1)
	{
		SendMessage(WindowConsole, LB_DELETESTRING, 0, 0);
		iNext--;
	}

	const int uTextLen = (int)_tcslen(szText);
	TCHAR* szBuffer = new TCHAR[11 + uTextLen + 1];
	time_t now_time_t = time(NULL);
	struct tm* now_tm = localtime(&now_time_t);
	_tcsftime(szBuffer, 12, TEXT("%H:%M:%S   "), now_tm);
	memcpy(szBuffer + 11, szText, uTextLen * sizeof(TCHAR));
	szBuffer[11 + uTextLen] = TEXT('\0');

		::SendMessage(WindowConsole, LB_INSERTSTRING, iNext, (LPARAM)szBuffer);
		::SendMessage(WindowConsole, LB_SETSEL, FALSE, -1);
		::SendMessage(WindowConsole, LB_SETSEL, TRUE, iNext);
		::SendMessage(WindowConsole, LB_SETTOPINDEX, iNext, 0);
	iNext++;

		delete[] szBuffer;

	return TRUE;
}


void PopupMenu()
{
	if (!pupup_menu)
		return;

	POINT p;
	GetCursorPos(&p);

	BOOL iIndex = TrackPopupMenu(pupup_menu,			  // HMENU hMenu
		TPM_LEFTALIGN | 		 // UINT uFlags
			TPM_TOPALIGN |  	 // .
			TPM_NONOTIFY |  	 // .
			TPM_RETURNCMD | 	 // .
			TPM_RIGHTBUTTON,	 // .
		p.x,					 // int x
		p.y,					 // int y
		0,  					 // int nReserved
		WindowConsole,  		 // HWND hWnd
		NULL					 // HWND prcRect
	);


	if (iIndex)
	{
		switch (iIndex)
		{
			case MENU_ITEM_COPY:
			{
				CString paste="";
				char tmp[512]="";
				int maxTaille=0;
				//if(iNext%50==0)
				if (OpenClipboard(WindowConsole))
				{
					if (EmptyClipboard())
					{
						HANDLE hData;
						for(int i=0;i<iNext;i++) {
							::SendMessage(WindowConsole, LB_GETTEXT, i, (LPARAM)tmp);
							if(strlen(tmp)<510) {
								int pos = strlen(tmp);
								tmp[pos] = '\n';
								tmp[pos+1]=0;
							} else 
							{
								tmp[510] = '\n';
								tmp[511]=0;
							}
							paste += tmp;
							maxTaille+=strlen(tmp);
						}
						hData = GlobalAlloc(GHND, maxTaille + 5);
						LPTSTR pData;
						pData = (LPTSTR)GlobalLock(hData);
						lstrcpy((LPTSTR)pData, paste);
						GlobalUnlock(hData);
						if (::SetClipboardData(CF_TEXT, hData) == NULL)
							GlobalFree(hData);
					}
					CloseClipboard();
				}
				break;
			}
			case MENU_ITEM_CLEAR:
				while(iNext>0) {
					::SendMessage(WindowConsole, LB_DELETESTRING, --iNext, NULL);
				}
				break;
			case MENU_ITEM_CLOSE:
				pConsole->Close();
				break;
		}
	}
}

BOOL Console::Popup()
{
	if (!WindowConsole)
		return false;	
	if (!IsWindowVisible(WindowConsole))
	{
		ShowWindow(WindowConsole, SW_SHOW);
	}

	SetActiveWindow(WindowConsole);

	return TRUE;
}

BOOL Console::IsVisible()
{
	if (!WindowConsole)
		return FALSE;	
	return IsWindowVisible(WindowConsole);
}

BOOL Console::Close()
{
	if (!WindowConsole)
		return false;	
	if (IsWindowVisible(WindowConsole))
	{
		ShowWindow(WindowConsole, SW_HIDE);
		AfxGetApp()->WriteProfileInt(_T("Preferences"), _T("displayConsole"), 0);
	}

	return TRUE;
}

LRESULT CALLBACK WndprocConsole(HWND hwnd, UINT message, WPARAM wp, LPARAM lp)
{
	static BOOL bContextOpen = FALSE;

	switch (message)
	{
	case WM_CONTEXTMENU:
		{
			if (bContextOpen)
				break;
			bContextOpen = TRUE;

			PopupMenu();

			bContextOpen = FALSE;
			break;
		}

	case WM_GETMINMAXINFO:
		{
			MINMAXINFO* mmi = (MINMAXINFO*)lp;
			mmi->ptMinTrackSize.x = rConsoleDefault.right - rConsoleDefault.left;
			mmi->ptMinTrackSize.y = rConsoleDefault.bottom - rConsoleDefault.top;
			return 0;
		}

	case WM_SYSCOMMAND:
		if ((wp & 0xFFF0) == SC_CLOSE)
		{
			ShowWindow(hwnd, SW_HIDE);
			return 0;
		}
		break;

	case WM_DESTROY:
		break;

	case WM_SHOWWINDOW:
		bConsoleVisible = (wp == TRUE);
		break;
	}
	return CallWindowProc(WndprocConsoleBackup, hwnd, message, wp, lp);
}
